#!/usr/bin/env python

import os
NAME = 'Find_S2'
AUTHORS = 'Joyce Reimer, Kyle Klenk, Raymond Spiteri'
GUIinclude = True

from datetime import date
from carputils import settings
from carputils import tools
from carputils import mesh
from carputils.carpio import txt
import numpy as np
from numpy import array as nplist

def parser_for_s2():
    parser = tools.standard_parser()
    group  = parser.add_argument_group('experiment specific options')
    group.add_argument('--duration',
                        type = float,
                        default = 300.,
                        help = 'Duration of simulation in [ms] (default: 20.)')
    group.add_argument('--S1-strength',
                       type = float,
                       default = 172,
                       help = 'pick S1 transmembrane current stimulus strength in [uA/cm^2] = [pA/pF] considering fixed Cm (default: 20.)')
    group.add_argument('--S1-dur',
                        type = float,
                        default = 2.,
                        help = 'pick transmembrane current stimulus duration in [ms] (default: 2.)')
    group.add_argument('--S1-start',
                        type = float,
                        default = 0.0,
                        help = 'pick S1 start time')
    group.add_argument('--S2-strength',
                       type = float,
                       default = 200.,
                       help = 'pick S2 transmembrane current stimulus strength in [uA/cm^2] = [pA/pF] considering fixed Cm (default: 20.)')
    group.add_argument('--S2-dur',
                        type = float,
                        default = 2.,
                        help = 'pick transmembrane current stimulus duration in [ms] (default: 2.)')
    group.add_argument('--S2-start',
                        type = float,
                        default = 150.0,
                        help = 'pick S2 start time')
    return parser


def jobID(args):
    """
    Generate name of top level output directory.
    """
    today = date.today()
    return 'simulations/output/s2_{}_attempt_{}'.format(int(args.S2_start), int(args.S2_strength))


@tools.carpexample(parser_for_s2, jobID)
def run(args, job):
    #===========================================
    # 1: Define the mesh: Units are mm
    #===========================================
    
    # Create a block with a thin z dimension
    x = 10.0 #(mm)
    y = 1.0  #(mm)
    z = 0.2  #(mm)
    geom = mesh.Block(size=(x,y,z))

    # Set fibre angle to 0, sheet angle to 0
    geom.set_fibres(0, 0, 90, 90) #endo, epi, sheet endo, sheet epicard

    # Finilaize the creation of the mesh
    meshname = mesh.generate(geom)

    #===========================================
    # 2: Define the ionic models & conductivities
    #===========================================

    # Query for element labels
    _, etags, _ = txt.read(meshname + '.elem')
    etags = np.unique(etags)

    IntraTags = etags[etags != 0].tolist()
    ExtraTags = etags.tolist().copy()

    #===========================================
    # 3: Configure the simulation
    #===========================================

    # Define the geometry of the stimulus at one end of the block
    # Units are um
    stim = ['-num_stim',             2,
            '-stimulus[0].name',    'S1', #index 0 because only one type of electrode
            '-stimulus[0].stimtype', 0, #0 means intracellular stim
            '-stimulus[0].strength', args.S1_strength, #uA/Cm^2
            '-stimulus[0].duration', args.S1_dur,
            '-stimulus[0].start',    args.S1_start,
            '-stimulus[1].name',    'S2',
            '-stimulus[1].stimtype', 0,
            '-stimulus[1].strength', args.S2_strength,
            '-stimulus[1].duration', args.S2_dur,
            '-stimulus[1].start',    args.S2_start]

    #electrode = mesh.block_boundary_condition(geom, 'stimulus', 0, 'x', True) #True refers to a BC between tissue and bath
    E1_lower_bound = nplist([ 0.0, 0.0, 0.0])
    E1_upper_bound = nplist([ 2.0, 0.3, 0.2])
    electrode = mesh.block_region(geom, 'stimulus', 0, E1_lower_bound, E1_upper_bound, False) # introudce a stimulus in the region defined by the lower and upper bounds

    #electrode = mesh.block_boundary_condition(geom, 'stimulus', 0, 'x', True) #True refers to a BC between tissue and bath
    E1_lower_bound = nplist([ 0.0, 0.0, 0.0])
    E1_upper_bound = nplist([ 2.0, 0.3, 0.2])
    electrode = mesh.block_region(geom, 'stimulus', 0, E1_lower_bound, E1_upper_bound, False) # introudce a stimulus in the region defined by the lower and upper bounds

    #===========================================
    # 4: Defining simulator options:
    #===========================================

    # Get basic command line, including solver options
    cmd = tools.carp_cmd(os.path.join('simulations/input/basic.par')) #cmd is a carp command

    cmd += ['-simID',    job.ID,
            '-meshname', meshname,
            '-dt',       25,
            '-tend',     args.duration]
    cmd += tools.gen_physics_opts(ExtraTags=ExtraTags, IntraTags=IntraTags)
    cmd += stim + electrode

    # Run simulation 
    print(cmd)
    job.carp(cmd) #cmd includes the params

run()